import { ApolloQueryResult } from "@apollo/client";
import { IApolloClient } from "@deep-foundation/hasura/client";
import React from "react";
import { Link, MinilinksResult } from "./minilinks";
import { BoolExpCan, BoolExpHandler, BoolExpLink, BoolExpSelector, BoolExpTree, BoolExpValue, MutationInputLink, MutationInputLinkPlain, MutationInputValue } from './client_types';
export declare const _ids: {
    '@deep-foundation/core': {
        [key: string]: number;
    };
};
export declare function parseJwt(token: any): {
    userId: number;
    role: string;
    roles: string[];
    [key: string]: any;
};
export interface DeepClientOptions<L = Link<number>> {
    linkId?: number;
    token?: string;
    handleAuth?: (linkId?: number, token?: string) => any;
    deep?: DeepClientInstance<L>;
    apolloClient?: IApolloClient<any>;
    minilinks?: MinilinksResult<L>;
    table?: string;
    returning?: string;
    selectReturning?: string;
    insertReturning?: string;
    updateReturning?: string;
    deleteReturning?: string;
    defaultSelectName?: string;
    defaultInsertName?: string;
    defaultUpdateName?: string;
    defaultDeleteName?: string;
    silent?: boolean;
}
export interface DeepClientResult<R> extends ApolloQueryResult<R> {
    error?: any;
}
export declare type DeepClientPackageSelector = string;
export declare type DeepClientPackageContain = string;
export declare type DeepClientLinkId = number;
export declare type DeepClientStartItem = DeepClientPackageSelector | DeepClientLinkId;
export declare type DeepClientPathItem = DeepClientPackageContain | boolean;
export interface DeepClientInstance<L = Link<number>> {
    linkId?: number;
    token?: string;
    handleAuth?: (linkId?: number, token?: string) => any;
    deep: DeepClientInstance<L>;
    apolloClient?: IApolloClient<any>;
    minilinks?: MinilinksResult<L>;
    table?: string;
    returning?: string;
    selectReturning?: string;
    insertReturning?: string;
    updateReturning?: string;
    deleteReturning?: string;
    defaultSelectName?: string;
    defaultInsertName?: string;
    defaultUpdateName?: string;
    defaultDeleteName?: string;
    stringify(any?: any): string;
    select<Table extends 'links' | 'numbers' | 'strings' | 'objects' | 'can' | 'selectors' | 'tree' | 'handlers', LL = L>(exp: (Table extends 'numbers' ? BoolExpValue<number> : Table extends 'strings' ? BoolExpValue<string> : Table extends 'objects' ? BoolExpValue<object> : Table extends 'can' ? BoolExpCan : Table extends 'selectors' ? BoolExpSelector : Table extends 'tree' ? BoolExpTree : Table extends 'handlers' ? BoolExpHandler : BoolExpLink) | number | number[], options?: {
        table?: Table;
        returning?: string;
        variables?: any;
        name?: string;
    }): Promise<DeepClientResult<LL[]>>;
    insert<Table extends 'links' | 'numbers' | 'strings' | 'objects', LL = L>(objects: (Table extends 'numbers' ? MutationInputValue<number> : Table extends 'strings' ? MutationInputValue<string> : Table extends 'objects' ? MutationInputValue<any> : MutationInputLink), options?: {
        table?: Table;
        returning?: string;
        variables?: any;
        name?: string;
    }): Promise<DeepClientResult<{
        id: any;
    }[]>>;
    update<Table extends 'links' | 'numbers' | 'strings' | 'objects' | 'can' | 'selectors' | 'tree' | 'handlers'>(exp: (Table extends 'numbers' ? BoolExpValue<number> : Table extends 'strings' ? BoolExpValue<string> : Table extends 'objects' ? BoolExpValue<object> : Table extends 'can' ? BoolExpCan : Table extends 'selectors' ? BoolExpSelector : Table extends 'tree' ? BoolExpTree : Table extends 'handlers' ? BoolExpHandler : BoolExpLink) | number | number[], value: (Table extends 'numbers' ? MutationInputValue<number> : Table extends 'strings' ? MutationInputValue<string> : Table extends 'objects' ? MutationInputValue<any> : MutationInputLinkPlain), options?: {
        table?: Table;
        returning?: string;
        variables?: any;
        name?: string;
    }): Promise<DeepClientResult<{
        id: any;
    }[]>>;
    delete<Table extends 'links' | 'numbers' | 'strings' | 'objects' | 'can' | 'selectors' | 'tree' | 'handlers'>(exp: (Table extends 'numbers' ? BoolExpValue<number> : Table extends 'strings' ? BoolExpValue<string> : Table extends 'objects' ? BoolExpValue<object> : Table extends 'can' ? BoolExpCan : Table extends 'selectors' ? BoolExpSelector : Table extends 'tree' ? BoolExpTree : Table extends 'handlers' ? BoolExpHandler : BoolExpLink) | number | number[], options?: {
        table?: Table;
        returning?: string;
        variables?: any;
        name?: string;
    }): Promise<DeepClientResult<{
        id: any;
    }[]>>;
    reserve<LL = L>(count: number): Promise<number[]>;
    await(id: number): Promise<boolean>;
    serializeWhere(exp: any, env?: string): any;
    serializeQuery(exp: any, env?: string): any;
    id(start: DeepClientStartItem, ...path: DeepClientPathItem[]): Promise<number>;
    idSync(start: DeepClientStartItem, ...path: DeepClientPathItem[]): number;
    guest(options: DeepClientGuestOptions): Promise<DeepClientAuthResult>;
    jwt(options: DeepClientJWTOptions): Promise<DeepClientAuthResult>;
    login(options: DeepClientJWTOptions): Promise<DeepClientAuthResult>;
    logout(): Promise<DeepClientAuthResult>;
    can(objectIds: number[], subjectIds: number[], actionIds: number[]): Promise<boolean>;
}
export interface DeepClientAuthResult {
    linkId?: number;
    token?: string;
    error?: any;
}
export interface DeepClientGuestOptions {
    relogin?: boolean;
}
export interface DeepClientJWTOptions {
    linkId?: number;
    token?: string;
    relogin?: boolean;
}
export declare class DeepClient<L = Link<number>> implements DeepClientInstance<L> {
    linkId?: number;
    token?: string;
    handleAuth?: (linkId?: number, token?: string) => any;
    deep: DeepClientInstance<L>;
    apolloClient?: IApolloClient<any>;
    minilinks?: MinilinksResult<L>;
    table?: string;
    returning?: string;
    selectReturning?: string;
    insertReturning?: string;
    updateReturning?: string;
    deleteReturning?: string;
    defaultSelectName?: string;
    defaultInsertName?: string;
    defaultUpdateName?: string;
    defaultDeleteName?: string;
    silent: boolean;
    _silent(options?: Partial<{
        silent?: boolean;
    }>): boolean;
    constructor(options: DeepClientOptions<L>);
    stringify(any?: any): string;
    select<Table extends 'links' | 'numbers' | 'strings' | 'objects' | 'can' | 'selectors' | 'tree' | 'handlers', LL = L>(exp: (Table extends 'numbers' ? BoolExpValue<number> : Table extends 'strings' ? BoolExpValue<string> : Table extends 'objects' ? BoolExpValue<object> : Table extends 'can' ? BoolExpCan : Table extends 'selectors' ? BoolExpSelector : Table extends 'tree' ? BoolExpTree : Table extends 'handlers' ? BoolExpHandler : BoolExpLink) | number | number[], options?: {
        table?: Table;
        returning?: string;
        variables?: any;
        name?: string;
    }): Promise<DeepClientResult<LL[]>>;
    insert<Table extends 'links' | 'numbers' | 'strings' | 'objects', LL = L>(objects: (Table extends 'numbers' ? MutationInputValue<number> : Table extends 'strings' ? MutationInputValue<string> : Table extends 'objects' ? MutationInputValue<any> : MutationInputLink), options?: {
        table?: Table;
        returning?: string;
        variables?: any;
        name?: string;
        silent?: boolean;
    }): Promise<DeepClientResult<{
        id: any;
    }[]>>;
    update<Table extends 'links' | 'numbers' | 'strings' | 'objects' | 'can' | 'selectors' | 'tree' | 'handlers'>(exp: (Table extends 'numbers' ? BoolExpValue<number> : Table extends 'strings' ? BoolExpValue<string> : Table extends 'objects' ? BoolExpValue<object> : Table extends 'can' ? BoolExpCan : Table extends 'selectors' ? BoolExpSelector : Table extends 'tree' ? BoolExpTree : Table extends 'handlers' ? BoolExpHandler : BoolExpLink) | number | number[], value: (Table extends 'numbers' ? MutationInputValue<number> : Table extends 'strings' ? MutationInputValue<string> : Table extends 'objects' ? MutationInputValue<any> : MutationInputLinkPlain), options?: {
        table?: Table;
        returning?: string;
        variables?: any;
        name?: string;
        silent?: boolean;
    }): Promise<DeepClientResult<{
        id: any;
    }[]>>;
    delete<Table extends 'links' | 'numbers' | 'strings' | 'objects' | 'can' | 'selectors' | 'tree' | 'handlers'>(exp: (Table extends 'numbers' ? BoolExpValue<number> : Table extends 'strings' ? BoolExpValue<string> : Table extends 'objects' ? BoolExpValue<object> : Table extends 'can' ? BoolExpCan : Table extends 'selectors' ? BoolExpSelector : Table extends 'tree' ? BoolExpTree : Table extends 'handlers' ? BoolExpHandler : BoolExpLink) | number | number[], options?: {
        table?: Table;
        returning?: string;
        variables?: any;
        name?: string;
        silent?: boolean;
    }): Promise<DeepClientResult<{
        id: any;
    }[]>>;
    reserve<LL = L>(count: number): Promise<number[]>;
    await(id: number, options?: {
        results: boolean;
    }): Promise<any>;
    _serialize: {
        link: {
            value: string;
            relations: {
                string: string;
                number: string;
                object: string;
                to: string;
                from: string;
                in: string;
                out: string;
                type: string;
                typed: string;
            };
        };
        value: {
            relations: {
                link: string;
            };
        };
    };
    _boolExpFields: {
        _and: boolean;
        _not: boolean;
        _or: boolean;
    };
    serializeWhere(exp: any, env?: string): any;
    serializeQuery(exp: any, env?: string): any;
    pathToWhere(start: DeepClientStartItem, ...path: DeepClientPathItem[]): any;
    id(start: DeepClientStartItem, ...path: DeepClientPathItem[]): Promise<number>;
    idSync(start: DeepClientStartItem, ...path: DeepClientPathItem[]): number;
    guest(options?: DeepClientGuestOptions): Promise<DeepClientAuthResult>;
    jwt(options: DeepClientJWTOptions): Promise<DeepClientAuthResult>;
    whoami(): Promise<number | undefined>;
    login(options: DeepClientJWTOptions): Promise<DeepClientAuthResult>;
    logout(): Promise<DeepClientAuthResult>;
    can(objectIds: number | number[], subjectIds: number | number[], actionIds: number | number[], userIds?: number | number[]): Promise<boolean>;
}
export declare const JWT: import("@apollo/client").DocumentNode;
export declare const WHOISME: import("@apollo/client").DocumentNode;
export declare const GUEST: import("@apollo/client").DocumentNode;
export declare function useAuthNode(): [number, (value: number) => any, () => any];
export declare const DeepContext: React.Context<DeepClient<Link<number>>>;
export declare function useDeepGenerator(apolloClientProps?: IApolloClient<any>): DeepClient<Link<number>>;
export declare function DeepProvider({ apolloClient: apolloClientProps, children, }: {
    apolloClient?: IApolloClient<any>;
    children: any;
}): JSX.Element;
export declare function useDeep(): DeepClient<Link<number>>;
export declare function useDeepQuery(query: any, options?: any): any;
